@@ -2,28 +2,28 @@
 #  All updates are stolen from survexp in the survival package, with comments.
 # Most changes are used, some further corrections were required.
 rformulate <- function (formula, data = parent.frame(), ratetable, na.action, rmap,
-                        int, centered, cause) 
-{ 
+                        int, centered, cause)
+{
   call <- match.call()
   m <- match.call(expand.dots = FALSE)
-  
+
   # keep the parts of the call that we want, toss others
   m <- m[c(1, match(c("formula", "data", "cause"), names(m), nomatch=0))]
   m[[1L]] <- quote(stats::model.frame)  # per CRAN, the formal way to set it
 
-  Terms <- if (missing(data)) 
-               terms(formula, specials= "ratetable")
-           else terms(formula, specials= "ratetable", data = data)
+  Terms <- if (missing(data))
+               terms(formula, specials= c("strata","ratetable"))
+           else terms(formula, specials=c("strata", "ratetable"), data = data)
   Term2 <- Terms
 
   #sorting out the ratetable argument - matching demographic variables
   rate <- attr(Terms, "specials")$ratetable
-  if (length(rate) > 1) 
+  if (length(rate) > 1)
     stop("Can have only 1 ratetable() call in a formula")
-  
+
   #matching demographic variables via rmap
   if (!missing(rmap)) {  # use this by preference
-      if (length(rate) >0) 
+      if (length(rate) >0)
           stop("cannot have both ratetable() in the formula and a rmap argument")
           rcall <- rmap
           if (!is.call(rcall) || rcall[[1]] != as.name('list'))
@@ -32,20 +32,20 @@ rformulate <- function (formula, data = parent.frame(), ratetable, na.action, rm
   else if (length(rate) >0) { #sorting out ratetable
       stemp <- untangle.specials(Terms, 'ratetable')
       rcall <- as.call(parse(text=stemp$var)[[1]])   # as a call object
-      rcall[[1]] <- as.name('list')                  # make it a call to list   
+      rcall[[1]] <- as.name('list')                  # make it a call to list
       Term2 <- Term2[-stemp$terms]                   # remove from the formula
   }
-  else rcall <- NULL  # A ratetable, but no rcall or ratetable() 
-    
+  else rcall <- NULL  # A ratetable, but no rcall or ratetable()
+
   # Check that there are no illegal names in rcall, then expand it
   #  to include all the names in the ratetable
   if (is.ratetable(ratetable))   {
       israte <- TRUE
       dimid <- names(dimnames(ratetable))
-      if (is.null(dimid)) 
+      if (is.null(dimid))
         dimid <- attr(ratetable, "dimid") # older style
      else  attr(ratetable, "dimid") <- dimid		#put all tables into the old style
-  
+
       temp <- match(names(rcall)[-1], dimid) # 2,3,... are the argument names
       if (any(is.na(temp)))
         stop("Variable not found in the ratetable:", (names(rcall))[is.na(temp)])
@@ -61,12 +61,12 @@ rformulate <- function (formula, data = parent.frame(), ratetable, na.action, rm
         }
   }
   else stop("invalid ratetable")
-   
+
   # Create a temporary formula, used only in the call to model.frame,
   #  that has extra variables
   newvar <- all.vars(rcall)
   if (length(newvar) > 0) {
-      tform <- paste(paste(deparse(Term2), collapse=""),  
+      tform <- paste(paste(deparse(Term2), collapse=""),
                      paste(newvar, collapse='+'), sep='+')
       m$formula <- as.formula(tform, environment(Terms))
   }
@@ -77,8 +77,8 @@ rformulate <- function (formula, data = parent.frame(), ratetable, na.action, rm
   Y <- model.extract(m, "response")
   offset <- model.offset(m)
   if (length(offset)==0) offset <- rep(0., n)
-  
-  if (!is.Surv(Y)) 
+
+  if (!is.Surv(Y))
     stop("Response must be a survival object")
   Y.surv <- Y
   if (attr(Y, "type") == "right") {
@@ -96,13 +96,13 @@ rformulate <- function (formula, data = parent.frame(), ratetable, na.action, rm
     ncol0 <- 3
   }
   else stop("Illegal response value")
-  if (any(c(Y, start) < 0)) 
+  if (any(c(Y, start) < 0))
     stop("Negative follow up time")
   if(max(Y)<30)
     warning("The event times must be expressed in days! (Your max time in the data is less than 30 days) \n")
 
   # rdata contains the variables matching the ratetable
-  rdata <- data.frame(eval(rcall, m), stringsAsFactors=TRUE)  
+  rdata <- data.frame(eval(rcall, m), stringsAsFactors=TRUE)
   rtemp <- match.ratetable(rdata, ratetable)		#this function puts the dates in R and in cutpoints in rtabledate
   R <- rtemp$R
   cutpoints <- rtemp$cutpoints
@@ -112,7 +112,7 @@ rformulate <- function (formula, data = parent.frame(), ratetable, na.action, rm
   attr(ratetable, "dimid") <- dimid
   rtorig <- attributes(ratetable)
   nrt <- length(rtorig$dimid)
- 
+
   #checking if the ratetable variables are given in days
   wh.age <- which(dimid=="age")
   wh.year <- which(dimid=="year")
@@ -122,7 +122,7 @@ rformulate <- function (formula, data = parent.frame(), ratetable, na.action, rm
   }
   # TMT -- note the new class
   if(length(wh.year)>0){
-      if(min(R[,wh.year])>1850 & max(R[,wh.year])<2020& 
+      if(min(R[,wh.year])>1850 & max(R[,wh.year])<2020&
          class(cutpoints[[wh.year]])=="rtdate")
         warning("The calendar year must be one of the date classes (Date, date, POSIXt)\n (Your variable seems to be expressed in years) \n")
   }
@@ -133,11 +133,11 @@ rformulate <- function (formula, data = parent.frame(), ratetable, na.action, rm
         if(rtorig$type[it]!=1)warning(paste("Variable ",rtorig$dimid[it]," is held fixed even though it changes in time in the population tables. \n (You may wish to set a value for each individual and not just one value for all)",sep=""))
     }
   }
-  
+
   #NEW in 2.05 (strata)
   # Now create the X matrix and strata
   strats <- attr(Term2, "specials")$strata
-  if (length(strats)) {  
+  if (length(strats)) {
       temp_str <- untangle.specials(Term2,"strata",1)
       if (length(temp_str$vars) == 1)
           strata.keep <- m[[temp_str$vars]]
@@ -158,7 +158,7 @@ rformulate <- function (formula, data = parent.frame(), ratetable, na.action, rm
   else mvalue <- double(mm)
 
   cause <- model.extract(m, "cause")
-  if(is.null(cause)) cause <- rep(2,nrow(m))					
+  if(is.null(cause)) cause <- rep(2,nrow(m))
 
   #NEW: ce cause manjka
   #status[cause==0] <- 0
@@ -169,7 +169,7 @@ rformulate <- function (formula, data = parent.frame(), ratetable, na.action, rm
     Y <- pmin(Y, int * 365.241)
     keep <- keep & (start < int * 365.241)
   }
-  if (any(start > Y) | any(Y < 0)) 
+  if (any(start > Y) | any(Y < 0))
     stop("Negative follow-up times")
 
   if (!all(keep)) {
@@ -177,7 +177,7 @@ rformulate <- function (formula, data = parent.frame(), ratetable, na.action, rm
       Y <- Y[keep]
       start <- start[keep]
       status <- status[keep]
-      R <- R[keep, ,drop=FALSE] 
+      R <- R[keep, ,drop=FALSE]
       strata.keep <- strata.keep[keep] # dodano za strato  #NEW in 2.05
       offset <- offset[keep]
       Y.surv <- Y.surv[keep, , drop = FALSE]
@@ -189,29 +189,29 @@ rformulate <- function (formula, data = parent.frame(), ratetable, na.action, rm
   # I do not want to preserve variable class here - so paste R onto here, give it names
 temp <- R
   names(temp) <- paste0("X", 1:ncol(temp))    # with the right names
-  
+
   #if variable class needs to be preserved, use this instead
   #  variable class.  So paste on rdata, but with the right order and names
   #temp <- rdata[,match(dimid, names(rdata))]  # in the right order
   #names(temp) <- paste0("X", 1:ncol(temp))    # with the right names
- 
+
  data <- data.frame(start = start, Y = Y, stat = status, temp)
   if (mm != 0) data <- cbind(data, X)
-  
+
   # we pass the altered cutpoints forward, keep them in the date format (could be changed eventually to get rid of the date package dependence)
   attr(ratetable, "cutpoints") <- lapply(cutpoints, function(x) {
-      if (class(x) == 'rtabledate') class(x) <- 'date'
+      if(inherits(x, 'rtabledate')) class(x) <- 'date'
       x})
 
-  out <- list(data = data, R = R, status = status, start = start, 
-            Y = Y, X = as.data.frame(X), m = mm, n = n, type = type, 
-            Y.surv = Y.surv, 
-            Terms = Terms, ratetable = ratetable, offset = offset, 
+  out <- list(data = data, R = R, status = status, start = start,
+            Y = Y, X = as.data.frame(X), m = mm, n = n, type = type,
+            Y.surv = Y.surv,
+            Terms = Terms, ratetable = ratetable, offset = offset,
             formula=formula,
             cause = cause, mvalue=mvalue, strata.keep=strata.keep) # dodano za strato  #NEW in 2.05
 
   na.action <- attr(m, "na.action")
-  if (length(na.action)) 
+  if (length(na.action))
     out$na.action <- na.action
   out
 }
